library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity pulse_counter is
    Generic (
        counter_width : integer := 16);
    Port (
        CLK_in : in STD_LOGIC;
        RST_in : in STD_LOGIC;
        pulse_in : in STD_LOGIC;
        count_out : out STD_LOGIC_VECTOR (counter_width-1 downto 0));
end pulse_counter;

architecture Behavioral of pulse_counter is
    constant counter_max : integer := 2**counter_width-1;
	signal prev_clk : STD_LOGIC := '0';
    signal counter: integer range 0 to counter_max := 0;
begin
        process(pulse_in, RST_in)
    begin
        if (rising_edge(pulse_in)) then
            if (RST_in = '0' OR counter = counter_max) then
                count_out <= (others => '0');
            elsif (clk_in = '1' and prev_clk = '0') then
                counter <= 1;
                count_out <= std_logic_vector(to_unsigned(1,counter_width));
            elsif (clk_in = '0' and prev_clk = '1') then
                count_out <=  std_logic_vector(to_unsigned(counter, counter_width));
            else
                counter <= counter + 1;  
            end if;                
            prev_clk <= clk_in;            
        end if;
    end process;

end Behavioral;
