library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity top_level is
    Generic (
        counter_width : integer := 16);
    Port (
        CLK_in : in STD_LOGIC;
        RST_in : in STD_LOGIC;
        EN_in : in STD_LOGIC;
        pulse_out : out STD_LOGIC;
        count_out : out STD_LOGIC_VECTOR (counter_width-1 downto 0));
end top_level;

architecture Behavioral of top_level is
    component logic_gate is
        Port (
            logic_in : in STD_LOGIC;
            logic_out : out STD_LOGIC);
    end component;
    
    component pulse_counter is
        Generic (
            counter_width : integer := 16);
        Port (
            CLK_in : in STD_LOGIC;
            RST_in : in STD_LOGIC;
            pulse_in : in STD_LOGIC;
            count_out : out STD_LOGIC_VECTOR (counter_width-1 downto 0));
    end component;
    

    
    constant stage_nr : integer := 3;
    signal pulse_buff : STD_LOGIC := '0';
	signal stage_array : std_logic_vector(0 to stage_nr);
begin
    pulse_counter_inst: pulse_counter port map (CLK_in, RST_in, pulse_buff, count_out);
    gen: for i in 0 to stage_nr-1 generate
        logic_gate_inst: logic_gate port map(stage_array(i), stage_array(i+1));
    end generate gen;
    stage_array(0) <= stage_array(stage_nr) AND EN_in;
    pulse_buff <= stage_array(stage_nr) AND EN_in;
    pulse_out <= pulse_buff;
    
end Behavioral;
